@@ -25,17 +25,13 @@
 
 #include <config.h>
 #include "constraints.h"
-#include "workspace-private.h"
+#include "workspace.h"
 #include "place.h"
+#include "prefs.h"
 
 #include <stdlib.h>
 #include <math.h>
 
-#define NOT_TOO_SMALL_BORDER 20    /* space around the resized window */
-#define NOT_TOO_SMALL_TRIGGER 0.60 /* percentage of screen size below which
-                                    * the NOT_TOO_SMALL constraint triggers
-                                    */
-
 #if 0
  // This is the short and sweet version of how to hack on this file; see
  // doc/how-constraints-works.txt for the gory details.  The basics of
@@ -106,8 +102,7 @@ typedef enum
   PRIORITY_SIZE_HINTS_LIMITS = 3,
   PRIORITY_TITLEBAR_VISIBLE = 4,
   PRIORITY_PARTIALLY_VISIBLE_ON_WORKAREA = 4,
-  PRIORITY_NOT_TOO_SMALL = 5,
-  PRIORITY_MAXIMUM = 5 /* Dummy value used for loop end = max(all priorities) */
+  PRIORITY_MAXIMUM = 4 /* Dummy value used for loop end = max(all priorities) */
 } ConstraintPriority;
 
 typedef enum
@@ -183,11 +178,6 @@ static gboolean constrain_partially_onscreen (MetaWindow         *window,
                                               ConstraintPriority  priority,
                                               gboolean            check_only);
 
-static gboolean constrain_not_too_small      (MetaWindow         *window,
-                                              ConstraintInfo     *info,
-                                              ConstraintPriority  priority,
-                                              gboolean            check_only);
-
 static void setup_constraint_info        (ConstraintInfo      *info,
                                           MetaWindow          *window,
                                           MetaFrameGeometry   *orig_fgeom,
@@ -229,8 +219,6 @@ static const Constraint all_constraints[] = {
   {constrain_fully_onscreen,     "constrain_fully_onscreen"},
   {constrain_titlebar_visible,   "constrain_titlebar_visible"},
   {constrain_partially_onscreen, "constrain_partially_onscreen"},
-  {constrain_not_too_small,      "constrain_not_too_small"},
-
   {NULL,                         NULL}
 };
 
@@ -434,11 +422,11 @@ setup_constraint_info (ConstraintInfo      *info,
     meta_workspace_get_onxinerama_region (cur_workspace, 
                                           xinerama_info->number);
 
-#if 0
   /* Workaround braindead legacy apps that don't know how to
    * fullscreen themselves properly.
    */
-  if (meta_rectangle_equal (new, &xinerama_info->rect) &&
+  if (meta_prefs_get_force_fullscreen() &&
+      meta_rectangle_equal (new, &xinerama_info->rect) &&
       window->has_fullscreen_func &&
       !window->fullscreen)
     {
@@ -451,7 +439,6 @@ setup_constraint_info (ConstraintInfo      *info,
                   window->desc);
       meta_window_make_fullscreen_internal (window);
     }
-#endif
 
   /* Log all this information for debugging */
   meta_topic (META_DEBUG_GEOMETRY,
@@ -541,10 +528,11 @@ place_window_if_needed(MetaWindow     *window,
   if (window->placed || did_placement)
     {
       if (window->maximize_horizontally_after_placement ||
-          window->maximize_vertically_after_placement)
+          window->maximize_vertically_after_placement   ||
+          window->fullscreen_after_placement)
         {
-          /* define a sane saved_rect so that the user can unmaximize to
-           * something reasonable.
+          /* define a sane saved_rect so that the user can unmaximize or
+           * make unfullscreen to something reasonable.
            */
           if (info->current.width >= info->work_area_xinerama.width)
             {
@@ -571,6 +559,13 @@ place_window_if_needed(MetaWindow     *window,
           if (window->frame && !window->fullscreen)
             meta_frame_calc_geometry (window->frame, info->fgeom);
 
+          if (window->fullscreen_after_placement)
+            {
+              window->saved_rect = info->current;
+              window->fullscreen = TRUE;
+              window->fullscreen_after_placement = FALSE;
+            }
+
           window->maximize_horizontally_after_placement = FALSE;
           window->maximize_vertically_after_placement = FALSE;
         }
@@ -1385,89 +1380,3 @@ constrain_partially_onscreen (MetaWindow         *window,
 
   return retval;
 }
-
-static gboolean
-constrain_not_too_small (MetaWindow         *window,
-                         ConstraintInfo     *info,
-                         ConstraintPriority  priority,
-                         gboolean            check_only)
-{
-  gboolean already_satisfied = TRUE;
-  gboolean resize = FALSE;
-  gint screen_width, screen_height, width, height;
-  gint x = 0, y = 0, old_x = 0, old_y;
-  MetaRectangle *start_rect;
-  MetaRectangle min_size, max_size;
-
-  if (priority > PRIORITY_NOT_TOO_SMALL || /* None of our businesss */
-      info->is_user_action ||              /* Don't mess users about */
-      window->type != META_WINDOW_NORMAL)  /* App windows only */
-    return TRUE;
-
-  old_x = info->current.x - info->fgeom->left_width;
-  old_y = info->current.y - info->fgeom->top_height;
-
-      width  = info->current.width  +
-        info->fgeom->left_width + info->fgeom->right_width;
-  height = info->current.height +
-    info->fgeom->top_height + info->fgeom->bottom_height;
-
-  screen_width  = info->work_area_xinerama.width;
-  screen_height = info->work_area_xinerama.height;
-
-  if (width == screen_width && height == screen_height)
-    return TRUE;
-
-  if ((((gfloat)width / (gfloat) screen_width) > NOT_TOO_SMALL_TRIGGER))
-    {
-      already_satisfied = FALSE;
-      resize = TRUE;
-
-      x = NOT_TOO_SMALL_BORDER;
-      y = NOT_TOO_SMALL_BORDER;
-    }
-
-  if (!resize)
-    {
-      /*
-       * If we are not resizing the application, attempt to center it.
-       */
-      x = (screen_width  - info->current.width) / 2 - info->fgeom->left_width;
-      y = (screen_height - info->current.height) / 2 - info->fgeom->top_height;
-
-      if (x != old_x || y != old_y)
-        already_satisfied = FALSE;
-    }
-
-  if (check_only || already_satisfied)
-    return already_satisfied;
-
-  if (resize)
-    {
-  width = screen_width - 2 * NOT_TOO_SMALL_BORDER;
-  height = screen_height - 2 * NOT_TOO_SMALL_BORDER;
-
-  /* We respect the max size hints */
-  get_size_limits (window, info->fgeom, FALSE, &min_size, &max_size);
-
-  if (width > max_size.width)
-    width = max_size.width;
-
-  if (height > max_size.height)
-    height = max_size.height;
-    }
-
-  if (info->action_type == ACTION_MOVE_AND_RESIZE)
-    start_rect = &info->current;
-  else
-    start_rect = &info->orig;
-
-  start_rect->x      = x;
-  start_rect->y      = y;
-  start_rect->width  = width;
-  start_rect->height = height;
-
-  unextend_by_frame (start_rect, info->fgeom);
-
-  return TRUE;
-}
