@@ -24,8 +24,8 @@
 #ifndef META_PLACE_H
 #define META_PLACE_H
 
-#include "window.h"
-#include "frame.h"
+#include "window-private.h"
+#include "frame-private.h"
 
 void meta_window_place (MetaWindow *window,
                         MetaFrameGeometry *fgeom,
