@@ -43,11 +43,31 @@
 #include <X11/Xutil.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
+typedef struct _MetaGroup MetaGroup;
 typedef struct _MetaWindowQueue MetaWindowQueue;
 
 typedef gboolean (*MetaWindowForeachFunc) (MetaWindow *window,
                                            void       *data);
 
+typedef enum
+{
+  META_WINDOW_NORMAL,
+  META_WINDOW_DESKTOP,
+  META_WINDOW_DOCK,
+  META_WINDOW_DIALOG,
+  META_WINDOW_MODAL_DIALOG,
+  META_WINDOW_TOOLBAR,
+  META_WINDOW_MENU,
+  META_WINDOW_UTILITY,
+  META_WINDOW_SPLASHSCREEN
+} MetaWindowType;
+
+typedef enum
+{
+  META_MAXIMIZE_HORIZONTAL = 1 << 0,
+  META_MAXIMIZE_VERTICAL   = 1 << 1
+} MetaMaximizeFlags;
+
 typedef enum {
   META_CLIENT_TYPE_UNKNOWN = 0,
   META_CLIENT_TYPE_APPLICATION = 1,
@@ -65,8 +85,6 @@ typedef enum {
 
 struct _MetaWindow
 {
-  GObject parent_instance;
-  
   MetaDisplay *display;
   MetaScreen *screen;
   MetaWorkspace *workspace;
@@ -111,9 +129,6 @@ struct _MetaWindow
   /* Initial timestamp property */
   guint32 initial_timestamp;  
   
-  /* Whether this is an override redirect window or not */
-  guint override_redirect : 1;
-
   /* Whether we're maximized */
   guint maximized_horizontally : 1;
   guint maximized_vertically : 1;
@@ -129,6 +144,9 @@ struct _MetaWindow
   /* Whether we're fullscreen */
   guint fullscreen : 1;
 
+  /* Whether we have to fullscreen after placement */
+  guint fullscreen_after_placement : 1;
+
   /* Area to cover when in fullscreen mode.  If _NET_WM_FULLSCREEN_MONITORS has
    * been overridden (via a client message), the window will cover the union of
    * these monitors.  If not, this is the single monitor which the window's
@@ -160,11 +178,6 @@ struct _MetaWindow
    */
   guint mapped : 1;
   
-  /* Whether window has been hidden from view by lowering it to the bottom
-   * of window stack.
-   */
-  guint hidden : 1;
-
   /* Iconic is the state in WM_STATE; happens for workspaces/shading
    * in addition to minimize
    */
@@ -237,13 +250,16 @@ struct _MetaWindow
   /* Have we placed this window? */
   guint placed : 1;
 
+  /* Must we force_save_user_window_placement? */
+  guint force_save_user_rect : 1;
+
   /* Is this not a transient of the focus window which is being denied focus? */
   guint denied_focus_and_not_transient : 1;
 
   /* Has this window not ever been shown yet? */
   guint showing_for_first_time : 1;
 
-  /* Are we in meta_window_unmanage()? */
+  /* Are we in meta_window_free()? */
   guint unmanaging : 1;
 
   /* Are we in meta_window_new()? */
@@ -354,23 +370,9 @@ struct _MetaWindow
   
   /* Current dialog open for this window */
   int dialog_pid;
-  int dialog_pipe;
 
   /* maintained by group.c */
   MetaGroup *group;
-
-#ifdef HAVE_COMPOSITE_EXTENSIONS
-  GObject *compositor_private;
-#endif
-};
-
-struct _MetaWindowClass
-{
-  GObjectClass parent_class;
-
-  void (*workspace_changed) (MetaWindow *window, int  old_workspace);
-  void (*focus)             (MetaWindow *window);
-  void (*raised)            (MetaWindow *window);
 };
 
 /* These differ from window->has_foo_func in that they consider
@@ -396,16 +398,20 @@ MetaWindow* meta_window_new_with_attrs     (MetaDisplay *display,
                                             Window       xwindow,
                                             gboolean     must_be_viewable,
                                             XWindowAttributes *attrs);
-void        meta_window_unmanage           (MetaWindow  *window,
+void        meta_window_free               (MetaWindow  *window,
                                             guint32      timestamp);
 void        meta_window_calc_showing       (MetaWindow  *window);
 void        meta_window_queue              (MetaWindow  *window,
                                             guint queuebits);
 void        meta_window_minimize           (MetaWindow  *window);
 void        meta_window_unminimize         (MetaWindow  *window);
+void        meta_window_maximize           (MetaWindow        *window,
+                                            MetaMaximizeFlags  directions);
 void        meta_window_maximize_internal  (MetaWindow        *window,
                                             MetaMaximizeFlags  directions,
                                             MetaRectangle     *saved_rect);
+void        meta_window_unmaximize         (MetaWindow        *window,
+                                            MetaMaximizeFlags  directions);
 void        meta_window_make_above         (MetaWindow  *window);
 void        meta_window_unmake_above       (MetaWindow  *window);
 void        meta_window_shade              (MetaWindow  *window,
@@ -417,6 +423,11 @@ void        meta_window_change_workspace   (MetaWindow  *window,
 void        meta_window_stick              (MetaWindow  *window);
 void        meta_window_unstick            (MetaWindow  *window);
 
+void        meta_window_activate           (MetaWindow  *window,
+                                            guint32      current_time);
+void        meta_window_activate_with_workspace  (MetaWindow    *window,
+                                                  guint32        current_time,
+                                                  MetaWorkspace *workspace);   
 void        meta_window_make_fullscreen_internal (MetaWindow    *window);
 void        meta_window_make_fullscreen    (MetaWindow  *window);
 void        meta_window_unmake_fullscreen  (MetaWindow  *window);
@@ -448,6 +459,9 @@ void        meta_window_resize_with_gravity (MetaWindow  *window,
                                              int          gravity);
 
 
+/* Return whether the window would be showing if we were on its workspace */
+gboolean    meta_window_showing_on_its_workspace (MetaWindow *window);
+
 /* Return whether the window should be currently mapped */
 gboolean    meta_window_should_be_showing   (MetaWindow  *window);
 
@@ -484,6 +498,8 @@ void        meta_window_get_geometry         (MetaWindow  *window,
                                               int         *y,
                                               int         *width,
                                               int         *height);
+void        meta_window_get_outer_rect       (const MetaWindow *window,
+                                              MetaRectangle    *rect);
 void        meta_window_get_xor_rect         (MetaWindow          *window,
                                               const MetaRectangle *grab_wireframe_rect,
                                               MetaRectangle       *xor_rect);
@@ -584,6 +600,8 @@ gboolean meta_window_is_ancestor_of_transient (MetaWindow            *window,
 void     meta_window_foreach_ancestor         (MetaWindow            *window,
                                                MetaWindowForeachFunc  func,
                                                void                  *data);
+MetaWindow* meta_window_find_root_ancestor    (MetaWindow *window);
+
 
 void meta_window_begin_grab_op (MetaWindow *window,
                                 MetaGrabOp  op,
@@ -596,8 +614,12 @@ void meta_window_update_keyboard_move   (MetaWindow *window);
 
 void meta_window_update_layer (MetaWindow *window);
 
-void meta_window_recalc_features    (MetaWindow *window);
+gboolean meta_window_get_icon_geometry (MetaWindow    *window,
+                                        MetaRectangle *rect);
 
+const char* meta_window_get_startup_id (MetaWindow *window);
+
+void meta_window_recalc_features    (MetaWindow *window);
 void meta_window_recalc_window_type (MetaWindow *window);
 
 void meta_window_stack_just_below (MetaWindow *window,
